#include "stdafx.h"

#include <iostream>
#include <fstream>
#include <cstdlib>
#include <math.h>
#include <time.h>
#include <time.h>

using namespace std;

#define MAIN
#include "ldraw.hpp"

void pause(void)
{
	cout << "Press <Enter> to quit" << endl;
	getchar();
}
void usage(void)
{
	cout << "Slicerpro [options] InPattern, InFormer, Outfile" << endl;
	cout << "-x: Pattern and former in y-z plane, projection in x direction" << endl;
	cout << "-y: Pattern and former in x-z plane, projection in y direction (default)" << endl;
	cout << "-z: Pattern and former in x-y plane, projection in z direction" << endl;
	cout << "-n: No projection, only slicing" << endl;
	cout << "-c: Copy conditional lines from former file to output" << endl;
	cout << "-t: Don't retriangulate (debugging)" << endl;
	cout << "-m: Merge polys of uniform color facets" << endl;
	cout << "-ui: Don't unify vertices in input files" << endl;
	cout << "-uo: Don't unify vertices in output file" << endl;
	cout << "-ut <val>: Vertices unification threshold (default 0.005)" << endl;
	cout << "-sp: Expand subfiles in pattern" << endl;
	cout << "-sf: Expand subfiles in former" << endl;
	cout << "-l \"path\": define ldraw search path" << endl;
	cout << "-p <val>: precision (default 0.001)" << endl;
	pause();
}

int main(int argc, char *argv[])
{
	printf("\nLdraw Slicerpro 1.1 - by Philo");
	printf("\n------------------------------\n");

	if(argc<4) 
	{
		usage();
		return 1;
	}
	bool CondCopy=false;
	bool UnifyInput=true;
	bool UnifyOutput=true;

	LdPath* path=new LdPath();
	// Get input file path
	string curpath=argv[argc-2];
	size_t pathlen = curpath.find_last_of("\\");
	if(pathlen == string::npos) curpath = ".\\"; else curpath.resize(pathlen+1);
	path->addPath(curpath);
	string derivedpath=curpath;
	path->addPath(derivedpath.append("p\\"));
	derivedpath=curpath;
	path->addPath(derivedpath.append("s\\"));

	for(int i=1; i<argc-3; i++)
	{
		if(argv[i][0] != '-')
		{
			usage();
			return 1;
		}
		switch (argv[i][1])
		{
		case 'x':
			xdir=true;
			zdir=false;
			break;
		case 'y':
			xdir=false;
			zdir=false;
			break;
		case 'z':
			xdir=false;
			zdir=true;
			break;
		case 'p':
			i++;
			sscanf(argv[i], "%lf",&LdPrecision);
			break;
		case 'n':
			NoProject=true;
			break;
		case 'm':
			Merge=true;
			break;
		case 't':
			NoReTriangulate=true;
			break;
		case 'c':
			CondCopy=true;
			break;
		case 'u':
			switch (argv[i][2])
			{
			case 'i':
				UnifyInput=false;
				break;
			case 'o':
				UnifyOutput=false;
				break;
			case 't':
				i++;
				sscanf(argv[i], "%lf",&UnifyDist);
				break;
			default:
				usage();
				return 1;
			}
			break;
		case 's':
			switch (argv[i][2])
			{
			case 'f':
				ExpandFormer=true;
				break;
			case 'p':
				ExpandPattern=true;
				break;
			default:
				usage();
				return 1;
			}
			break;

		case 'l':
			i++;
			curpath = argv[i];
			if(curpath.at(curpath.size()-1) != '\\') 
			{
				curpath.append("\\");
			}
			derivedpath=curpath;
			path->addPath(derivedpath.append("p\\"));
			derivedpath=curpath;
			path->addPath(derivedpath.append("parts\\"));
			path->addPath(derivedpath.append("s\\"));
			curpath.append("Unofficial\\");
			derivedpath=curpath;
			path->addPath(derivedpath.append("p\\"));
			derivedpath=curpath;
			path->addPath(derivedpath.append("parts\\"));
			path->addPath(derivedpath.append("s\\"));

			break;
		default:
			usage();
			return 1;
		}
	}

	cout << endl << "Precision: " << LdPrecision << endl;
	cout << "Vertices unification threshold: " << UnifyDist << endl;
	if(xdir) cout << "Working in x direction" << endl;
	else if(zdir) cout << "Working in z direction" << endl;
	else cout << "Working in y direction" << endl;

	Fldraw pattern;
	Fldraw form;
	cout << endl << "Reading pattern file " << argv[argc-3] << endl;
	if(!pattern.ldLoad (argv[argc-3]))
	{
		cout << argv[argc-3] << " not found" << endl;
		pause();
		return 2;
	}

	cout << endl << "Reading former file " << argv[argc-2] << endl;
	if(!form.ldLoad (argv[argc-2]))
	{
		cout << argv[argc-2] << " not found" << endl;
		pause();
		return 2;
	}


	if(ExpandFormer || ExpandPattern)
	{
		cout << endl << "LDraw subfiles search locations:\n" << path->toString().c_str() << endl;
	}
	if(ExpandFormer)
	{
		cout << "\nExpanding former subfiles\n";
		form.ldExpand(path);
		cout << endl;
	}
	if(ExpandPattern)
	{
		cout << "\nExpanding pattern subfiles\n";
		pattern.ldExpand(path);
		cout << endl;
	}


	if(UnifyInput)
	{
		cout << endl << "Unifying pattern file vertices" << endl;
		pattern.Unify();
		cout << "Unifying former file vertices" << endl;
		form.Unify();
	}
	
	// Copy former file to keep 3D data.
	Fldraw form3D(form);


	if(xdir) 
	{
		form.swapXY();
		form3D.swapXY();
		pattern.swapXY();
	}
	if(zdir)
	{
		form.swapYZ();
		form3D.swapYZ();
		pattern.swapYZ();
	}

	// force all windings and flatten.
	// swapping xy or yz flips orientation of quads/tris

	form.ForceYOrientation();
	pattern.ForceYOrientation();

	Fldraw* output;	
	cout << endl << "Slicing "; 
	cout << (NoProject ? "":"and Projecting ");
	cout << "pattern" << endl;
	output = pattern.CutAll(form, form3D);


	if(CondCopy) output->CopyCond(form3D);


	if(xdir) 
	{
		output->swapXY();
		output->InvertWinding();
	}
	if(zdir)
	{
		output->swapYZ();
		output->InvertWinding();
	}


	if(UnifyOutput)
	{
		cout << endl << "Unifying output file vertices" << endl;
		output->Unify();
	}
	cout << endl << "Writing output file " << argv[argc-1] << endl;
	if(!output->ldSave(argv[argc-1]))
	{
		cout <<" Can't create" << argv[argc-1] << endl;
		pause();
		return 2;
	}
	pause();
	return 0;
}

